DEFAULT_RESULTS_LIMIT = 100
# Duplicated to keep API compatible until version 4.0
# Merge these two literals and mixins on version 4.0
SEARCH_MODEL_NAME_API_KWARG = 'search_model'
SEARCH_MODEL_NAME_KWARG = 'search_model_name'
TASK_RETRY_DELAY = 5
