<style>
    .appearance-list-column-sort-icon {
        /* Nullify the 5px padding to the right */
        margin-right: -5px;
    }

    .appearance-list-column-label {
        /* Make column text behave the same way as column links */
        padding: 0px;
    }

    .appearance-list-columns-help-text {
        margin-left: 3px;
    }

    .fashadow svg {
        color:rgba(0, 0, 0, 0.3);
        stroke: rgba(255, 255, 255, 0.3);
        stroke-width: 20;
        transform: translate(3px, 5px) !important;
    }
</style>
